@@ -5,11 +5,8 @@
 #include <cstdlib>   // malloc
 #include <cstring>   // memcpy
 #include <utility>   // swap
-#undef do_open       // fix perl fuckups
-#undef do_close      // fix perl fuckups
 #include <ostream>
 #include <istream>
-#include <algorithm> // min,max
 #include <stdexcept>
 #include <panda/iterator.h>
 
@@ -18,27 +15,6 @@ namespace panda {
 using std::size_t;
 
 class string {
-private:
-    union {
-        char*       buf; // heap pointer (possibly shared)
-        const char* ptr; // external pointer
-    } _u;
-    size_t _capacity; // _u.buf capacity. external pointer mode if capacity == 0
-    size_t _length;   // external/buffer string length
-
-    size_t* _buf_refcnt_ptr () { return (size_t*)(_u.buf - sizeof(size_t)); }
-    size_t  _buf_refcnt     () { return *_buf_refcnt_ptr(); }
-    size_t  _buf_refcnt_dec () { return --*_buf_refcnt_ptr(); }
-    size_t  _buf_refcnt_inc () { return ++*_buf_refcnt_ptr(); }
-    void    _buf_release    () { if (_capacity && _buf_refcnt_dec() <= 0) free(_buf_refcnt_ptr()); }
-
-    void _realloc (size_t size) {
-        char* heap = (char*)std::realloc(_u.buf - sizeof(size_t), size + sizeof(size_t) + 1);
-        if (!heap) throw std::bad_alloc();
-        _u.buf = heap + sizeof(size_t);
-        _capacity = size;
-    }
-
 public:
     typedef base_iterator<char*>                  iterator;
     typedef base_iterator<const char*>            const_iterator;
@@ -359,7 +335,7 @@ public:
     }
     size_t rfind (const char* p, size_t pos, size_t n) const {
         if (n <= _length) {
-            pos = std::min(_length - n, pos);
+            if (pos > _length - n) pos = _length - n;
             do {
                 if (std::strncmp(_u.ptr + pos, p, n) == 0) return pos;
             } while (pos-- > 0);
@@ -385,6 +361,27 @@ public:
     ~string () {
         _buf_release();
     }
+
+private:
+    union {
+        char*       buf; // heap pointer (possibly shared)
+        const char* ptr; // external pointer
+    } _u;
+    size_t _capacity; // _u.buf capacity. external pointer mode if capacity == 0
+    size_t _length;   // external/buffer string length
+
+    size_t* _buf_refcnt_ptr () { return (size_t*)(_u.buf - sizeof(size_t)); }
+    size_t  _buf_refcnt     () { return *_buf_refcnt_ptr(); }
+    size_t  _buf_refcnt_dec () { return --*_buf_refcnt_ptr(); }
+    size_t  _buf_refcnt_inc () { return ++*_buf_refcnt_ptr(); }
+    void    _buf_release    () { if (_capacity && _buf_refcnt_dec() <= 0) free(_buf_refcnt_ptr()); }
+
+    void _realloc (size_t size) {
+        char* heap = (char*)std::realloc(_u.buf - sizeof(size_t), size + sizeof(size_t) + 1);
+        if (!heap) throw std::bad_alloc();
+        _u.buf = heap + sizeof(size_t);
+        _capacity = size;
+    }
 };
 
 inline string operator+ (const string& lhs, const string& rhs) { return string(lhs).append(rhs); }
